﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;

namespace HeuristicLab.Problems.Instances {
  public abstract class ProblemInstanceProvider<T> : IProblemInstanceProvider<T> {
    private IProblemInstanceConsumer<T> Consumer { get; set; }

    public abstract string Name { get; }
    public abstract string Description { get; }
    public abstract Uri WebLink { get; }
    public abstract string ReferencePublication { get; }

    public bool ConsumerCanBeFed {
      get { return Consumer != null; }
    }

    public void SetConsumer(IProblemInstanceConsumer consumer) {
      if (consumer is IProblemInstanceConsumer<T>)
        Consumer = (IProblemInstanceConsumer<T>)consumer;
      else Consumer = null;
    }

    public bool FeedConsumer(IInstanceDescriptor descriptor) {
      return Consumer.LoadFrom(GetInstance(descriptor));
    }

    public abstract IEnumerable<IInstanceDescriptor> GetInstanceDescriptors();

    public abstract T GetInstance(IInstanceDescriptor descriptor);

  }
}
